@@ -121,15 +121,17 @@ static void do_setattr(char *path, struct bch_opt_strs opts, bool remove_all)
 static void setattr_usage(void)
 {
 	puts("bcachefs set-file-option - set attributes on files in a bcachefs filesystem\n"
-	     "Usage: bcachefs set-file-option [OPTIONS]... <files>\n"
+	     "Usage: bcachefs set-file-option [OPTION]... <files>\n"
 	     "\n"
 	     "Options:");
 
 	bch2_opts_usage(OPT_INODE);
-	puts("      --remove-all            Remove all file options\n"
-	     "                              To remove specific options, use: --option=-\n"
-	     "      -h                      Display this help and exit\n"
+	puts("      --remove-all             Remove all file options\n"
+	     "                               To remove specific options, use: --option=-\n"
+	     "  -h, --help                   Display this help and exit\n"
+	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
+	exit(EXIT_SUCCESS);
 }
 
 int cmd_setattr(int argc, char *argv[])
@@ -138,6 +140,14 @@ int cmd_setattr(int argc, char *argv[])
 	bool remove_all = false;
 
 	for (i = 1; i < argc; i++) {
+		if (strcmp(argv[i], "--help") == 0) {
+			setattr_usage();
+			exit(EXIT_SUCCESS);
+		}
+		if (strcmp(argv[i], "-h") == 0) {
+			setattr_usage();
+			exit(EXIT_SUCCESS);
+		}
 		if (strcmp(argv[i], "--remove-all") == 0) {
 			remove_all = true;
 			bch_remove_arg_from_argv(&argc, argv, i);
