@@ -11,6 +11,7 @@
 #include "cmds.h"
 #include "libbcachefs.h"
 #include "tools-util.h"
+#include "version.h"
 
 #include "bcachefs.h"
 
@@ -301,7 +302,8 @@ static void bcachefs_fuse_unlink(fuse_req_t req, fuse_ino_t dir_ino,
 	int ret = bch2_trans_commit_do(c, NULL, NULL,
 				BCH_TRANS_COMMIT_no_enospc,
 			    bch2_unlink_trans(trans, dir, &dir_u,
-					      &inode_u, &qstr, false));
+					      (subvol_inum) {}, &inode_u,
+					      &qstr, false));
 
 	fuse_reply_err(req, -ret);
 }
@@ -1213,7 +1215,7 @@ int cmd_fusemount(int argc, char *argv[])
 	if (fuse_opts.show_version) {
 		printf("FUSE library version %s\n", fuse_pkgversion());
 		fuse_lowlevel_version();
-		printf("bcachefs version: %s\n", VERSION_STRING);
+		printf("bcachefs version: %s\n", bcachefs_version);
 		ret = 0;
 		goto out;
 	}
