@@ -526,6 +526,7 @@ static void image_create_usage(void)
 	     "  -h, --help                   Display this help and exit\n"
 	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
+	exit(EXIT_SUCCESS);
 }
 
 static int cmd_image_create(int argc, char *argv[])
@@ -757,6 +758,7 @@ static void image_update_usage(void)
 	     "  -h, --help                   Display this help and exit\n"
 	     "\n"
 	     "Report bugs to <linux-bcachefs@vger.kernel.org>");
+	exit(EXIT_SUCCESS);
 }
 
 static int cmd_image_update(int argc, char *argv[])
@@ -809,14 +811,14 @@ static int cmd_image_update(int argc, char *argv[])
 static int image_usage(void)
 {
 	puts("bcachefs image - commands for creating and updating image files\n"
-	     "Usage: bcachefs image <CMD> [OPTION]...\n"
+	     "Usage: bcachefs image <create|update> [OPTION]... <file>\n"
             "\n"
             "Commands:\n"
             "  create                       Create a minimally-sized disk image\n"
             "  update                       Update a disk image, minimizing changes\n"
             "\n"
             "Report bugs to <linux-bcachefs@vger.kernel.org>");
-	return 0;
+	exit(EXIT_SUCCESS);
 }
 
 int image_cmds(int argc, char *argv[])
