@@ -55,6 +55,7 @@ static int bch2_btree_insert_node(struct btree_update *, struct btree_trans *,
 
 static int btree_node_topology_err(struct bch_fs *c, struct btree *b, struct printbuf *out)
 {
+	prt_printf(out, "in parent node:\n");
 	bch2_btree_pos_to_text(out, c, b);
 	prt_newline(out);
 	int ret = __bch2_topology_error(c, out);
@@ -66,7 +67,7 @@ static int btree_node_topology_err(struct bch_fs *c, struct btree *b, struct pri
 /*
  * Verify that child nodes correctly span parent node's range:
  */
-int bch2_btree_node_check_topology(struct btree_trans *trans, struct btree *b)
+int bch2_btree_node_check_topology_msg(struct btree_trans *trans, struct btree *b, struct printbuf *out)
 {
 	struct bch_fs *c = trans->c;
 	struct bpos node_min = b->key.k.type == KEY_TYPE_btree_ptr_v2
@@ -89,23 +90,21 @@ int bch2_btree_node_check_topology(struct btree_trans *trans, struct btree *b)
 	 */
 	if (b == btree_node_root(c, b)) {
 		if (!bpos_eq(b->data->min_key, POS_MIN)) {
-			CLASS(bch_log_msg, msg)(c);
-			prt_printf(&msg.m, "btree root with incorrect min_key: ");
-			bch2_bpos_to_text(&msg.m, b->data->min_key);
-			prt_newline(&msg.m);
+			prt_printf(out, "btree root with incorrect min_key: ");
+			bch2_bpos_to_text(out, b->data->min_key);
+			prt_newline(out);
 
-			bch2_count_fsck_err(c, btree_root_bad_min_key, &msg.m);
-			return btree_node_topology_err(c, b, &msg.m);
+			bch2_count_fsck_err(c, btree_root_bad_min_key, out);
+			return btree_node_topology_err(c, b, out);
 		}
 
 		if (!bpos_eq(b->data->max_key, SPOS_MAX)) {
-			CLASS(bch_log_msg, msg)(c);
-			prt_printf(&msg.m, "btree root with incorrect max_key: ");
-			bch2_bpos_to_text(&msg.m, b->data->max_key);
-			prt_newline(&msg.m);
+			prt_printf(out, "btree root with incorrect max_key: ");
+			bch2_bpos_to_text(out, b->data->max_key);
+			prt_newline(out);
 
-			bch2_count_fsck_err(c, btree_root_bad_max_key, &msg.m);
-			return btree_node_topology_err(c, b, &msg.m);
+			bch2_count_fsck_err(c, btree_root_bad_max_key, out);
+			return btree_node_topology_err(c, b, out);
 		}
 	}
 
@@ -124,16 +123,15 @@ int bch2_btree_node_check_topology(struct btree_trans *trans, struct btree *b)
 			: bpos_successor(prev.k->k.p);
 
 		if (!bpos_eq(expected_min, bp.v->min_key)) {
-			CLASS(bch_log_msg, msg)(c);
-			prt_str(&msg.m, "end of prev node doesn't match start of next node");
-			prt_str(&msg.m, "\nprev ");
-			bch2_bkey_val_to_text(&msg.m, c, bkey_i_to_s_c(prev.k));
-			prt_str(&msg.m, "\nnext ");
-			bch2_bkey_val_to_text(&msg.m, c, k);
-			prt_newline(&msg.m);
+			prt_str(out, "end of prev node doesn't match start of next node");
+			prt_str(out, "\nprev ");
+			bch2_bkey_val_to_text(out, c, bkey_i_to_s_c(prev.k));
+			prt_str(out, "\nnext ");
+			bch2_bkey_val_to_text(out, c, k);
+			prt_newline(out);
 
-			bch2_count_fsck_err(c, btree_node_topology_bad_min_key, &msg.m);
-			return btree_node_topology_err(c, b, &msg.m);
+			bch2_count_fsck_err(c, btree_node_topology_bad_min_key, out);
+			return btree_node_topology_err(c, b, out);
 		}
 
 		bch2_bkey_buf_reassemble(&prev, k);
@@ -141,25 +139,31 @@ int bch2_btree_node_check_topology(struct btree_trans *trans, struct btree *b)
 	}
 
 	if (bkey_deleted(&prev.k->k)) {
-		CLASS(bch_log_msg, msg)(c);
-		prt_printf(&msg.m, "empty interior node\n");
-		bch2_count_fsck_err(c, btree_node_topology_empty_interior_node, &msg.m);
-		return btree_node_topology_err(c, b, &msg.m);
+		prt_printf(out, "empty interior node\n");
+		bch2_count_fsck_err(c, btree_node_topology_empty_interior_node, out);
+		return btree_node_topology_err(c, b, out);
 	}
 
 	if (!bpos_eq(prev.k->k.p, b->key.k.p)) {
-		CLASS(bch_log_msg, msg)(c);
-		prt_str(&msg.m, "last child node doesn't end at end of parent node\nchild: ");
-		bch2_bkey_val_to_text(&msg.m, c, bkey_i_to_s_c(prev.k));
-		prt_newline(&msg.m);
+		prt_str(out, "last child node doesn't end at end of parent node\nchild: ");
+		bch2_bkey_val_to_text(out, c, bkey_i_to_s_c(prev.k));
+		prt_newline(out);
 
-		bch2_count_fsck_err(c, btree_node_topology_bad_max_key, &msg.m);
-		return btree_node_topology_err(c, b, &msg.m);
+		bch2_count_fsck_err(c, btree_node_topology_bad_max_key, out);
+		return btree_node_topology_err(c, b, out);
 	}
 
 	return 0;
 }
 
+int bch2_btree_node_check_topology(struct btree_trans *trans, struct btree *b)
+{
+	CLASS(bch_log_msg, msg)(trans->c);
+	msg.m.suppress = true;
+
+	return bch2_btree_node_check_topology_msg(trans, b, &msg.m);
+}
+
 /* Calculate ideal packed bkey format for new btree nodes: */
 
 static void __bch2_btree_calc_format(struct bkey_format_state *s, struct btree *b)
@@ -301,7 +305,7 @@ static bool can_use_btree_node(struct bch_fs *c,
 	if (target && !bch2_bkey_in_target(c, k, target))
 		return false;
 
-	unsigned durability = bch2_bkey_durability(c, k);
+	unsigned durability = bch2_btree_ptr_durability(c, k);
 
 	if (durability >= res->nr_replicas)
 		return true;
@@ -639,6 +643,7 @@ static void btree_update_new_nodes_mark_sb(struct btree_update *as)
 {
 	struct bch_fs *c = as->c;
 
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 	bool write_sb = false;
 	darray_for_each(as->new_nodes, i)
@@ -650,8 +655,10 @@ static void btree_update_new_nodes_mark_sb(struct btree_update *as)
 
 static void bkey_strip_reconcile(const struct bch_fs *c, struct bkey_s k)
 {
-	bool dropped;
+	if (bkey_deleted(k.k))
+		return;
 
+	bool dropped;
 	do {
 		dropped = false;
 
@@ -666,7 +673,9 @@ static void bkey_strip_reconcile(const struct bch_fs *c, struct bkey_s k)
 			}
 	} while (dropped);
 
-	bch2_bkey_drop_ptrs(k, p, entry, p.ptr.dev == BCH_SB_MEMBER_INVALID);
+	bch2_bkey_drop_ptrs_noerror(k, p, entry, p.ptr.dev == BCH_SB_MEMBER_INVALID);
+
+	BUG_ON(!bch2_bkey_nr_dirty_ptrs(c, k.s_c));
 }
 
 static bool bkey_has_reconcile(const struct bch_fs *c, struct bkey_s_c k)
@@ -1205,13 +1214,11 @@ bch2_btree_update_start(struct btree_trans *trans, struct btree_path *path,
 
 	BUG_ON(!path->should_be_locked);
 
-	if (watermark == BCH_WATERMARK_copygc)
-		watermark = BCH_WATERMARK_btree_copygc;
-	if (watermark < BCH_WATERMARK_btree)
-		watermark = BCH_WATERMARK_btree;
-
-	commit_flags &= ~BCH_WATERMARK_MASK;
-	commit_flags |= watermark;
+	if (watermark == BCH_WATERMARK_stripe) {
+		watermark = BCH_WATERMARK_normal;
+		commit_flags &= ~BCH_WATERMARK_MASK;
+		commit_flags |= watermark;
+	}
 
 	if (watermark < BCH_WATERMARK_reclaim &&
 	    journal_low_on_space(&c->journal)) {
@@ -1316,6 +1323,7 @@ bch2_btree_update_start(struct btree_trans *trans, struct btree_path *path,
 				      false,
 				      &devs_have,
 				      as->disk_res.nr_replicas,
+				      as->disk_res.nr_replicas,
 				      watermark,
 				      write_flags);
 	ret = PTR_ERR_OR_ZERO(req);
@@ -1343,8 +1351,17 @@ bch2_btree_update_start(struct btree_trans *trans, struct btree_path *path,
 			if (!bch2_err_matches(ret, BCH_ERR_operation_blocked))
 				break;
 			bch2_trans_unlock(trans);
-			bch2_wait_on_allocator(c, &cl);
+			bch2_wait_on_allocator(c, watermark, &cl);
 		} while (1);
+
+		/*
+		 * Don't block with btree locks held
+		 *
+		 * It would be nice if we could remove closures from waitlists
+		 * without waking up the waitlist:
+		 */
+		if (closure_nr_remaining(&cl) > 1)
+			bch2_trans_unlock(trans);
 	}
 
 	if (ret) {
@@ -1492,20 +1509,23 @@ bch2_btree_insert_keys_interior(struct btree_update *as,
 	     insert = bkey_next(insert))
 		bch2_insert_fixup_btree_ptr(as, trans, path, b, &node_iter, insert);
 
-	int ret = bch2_btree_node_check_topology(trans, b);
+	CLASS(bch_log_msg, msg)(as->c);
+	msg.m.suppress = true;
+
+	int ret = bch2_btree_node_check_topology_msg(trans, b, &msg.m);
 	if (ret) {
-		CLASS(printbuf, buf)();
+		prt_str(&msg.m, "inserted keys\n");
 
-		for (struct bkey_i *k = keys->keys;
-		     k != insert;
-		     k = bkey_next(k)) {
-			bch2_bkey_val_to_text(&buf, trans->c, bkey_i_to_s_c(k));
-			prt_newline(&buf);
-		}
+		scoped_guard(printbuf_indent, &msg.m)
+			for (struct bkey_i *k = keys->keys;
+			     k != insert;
+			     k = bkey_next(k)) {
+				bch2_bkey_val_to_text(&msg.m, trans->c, bkey_i_to_s_c(k));
+				prt_newline(&msg.m);
+			}
 
-		bch2_fs_fatal_error(as->c, "%ps -> %s(): check_topology error %s: inserted keys\n%s",
-				    (void *) _RET_IP_, __func__, bch2_err_str(ret), buf.buf);
-		dump_stack();
+		bch2_prt_task_backtrace(&msg.m, current, 0, GFP_KERNEL);
+		bch2_fs_emergency_read_only(as->c, &msg.m);
 		return ret;
 	}
 
@@ -1885,7 +1905,7 @@ static int bch2_btree_insert_node(struct btree_update *as, struct btree_trans *t
 			   __func__, b->c.level);
 		bch2_btree_update_to_text(&msg.m, as);
 		bch2_btree_path_to_text(&msg.m, trans, path_idx, path);
-		bch2_fs_emergency_read_only2(c, &msg.m);
+		bch2_fs_emergency_read_only(c, &msg.m);
 		return -EIO;
 	}
 
@@ -1951,7 +1971,7 @@ split:
 
 int bch2_btree_split_leaf(struct btree_trans *trans,
 			  btree_path_idx_t path,
-			  unsigned flags)
+			  enum bch_trans_commit_flags flags)
 {
 	/* btree_split & merge may both cause paths array to be reallocated */
 	struct btree *b = path_l(trans->paths + path)->b;
@@ -1959,6 +1979,7 @@ int bch2_btree_split_leaf(struct btree_trans *trans,
 	unsigned l;
 	int ret = 0;
 
+
 	as = bch2_btree_update_start(trans, trans->paths + path,
 				     trans->paths[path].level,
 				     true, 0, flags, 0);
@@ -2044,7 +2065,7 @@ int bch2_btree_increase_depth(struct btree_trans *trans, btree_path_idx_t path,
 int __bch2_foreground_maybe_merge(struct btree_trans *trans,
 				  btree_path_idx_t path,
 				  unsigned level,
-				  unsigned flags,
+				  enum bch_trans_commit_flags flags,
 				  u64 *merge_count,
 				  enum btree_node_sibling sib)
 {
@@ -2200,6 +2221,9 @@ int __bch2_foreground_maybe_merge(struct btree_trans *trans,
 	bch2_btree_update_add_new_node(as, n);
 	six_unlock_write(&n->c.lock);
 
+	ret = bch2_btree_node_check_topology(trans, n);
+	BUG_ON(ret);
+
 	new_path = bch2_path_get_unlocked_mut(trans, btree, n->c.level, n->key.k.p);
 	six_lock_increment(&n->c.lock, SIX_LOCK_intent);
 	mark_btree_node_locked(trans, trans->paths + new_path, n->c.level, BTREE_NODE_INTENT_LOCKED);
@@ -2730,12 +2754,6 @@ void bch2_btree_updates_to_text(struct printbuf *out, struct bch_fs *c)
 		bch2_btree_update_to_text(out, as);
 }
 
-static bool bch2_btree_interior_updates_pending(struct bch_fs *c)
-{
-	guard(mutex)(&c->btree.interior_updates.lock);
-	return !list_empty(&c->btree.interior_updates.list);
-}
-
 bool bch2_btree_interior_updates_flush(struct bch_fs *c)
 {
 	bool ret = bch2_btree_interior_updates_pending(c);
