@@ -559,7 +559,7 @@ static noinline int bch2_write_drop_io_error_ptrs(struct bch_write_op *op)
 		n = bkey_next(src);
 
 		if (bkey_extent_is_direct_data(&src->k)) {
-			bch2_bkey_drop_ptrs(bkey_i_to_s(src), p, entry,
+			bch2_bkey_drop_ptrs_noerror(bkey_i_to_s(src), p, entry,
 				bch2_dev_io_failures(&op->wbio.failed, p.ptr.dev));
 
 			if (!bch2_bkey_nr_dirty_ptrs(c, bkey_i_to_s_c(src)))
@@ -1059,7 +1059,8 @@ static int bch2_write_extent(struct bch_write_op *op, struct write_point *wp,
 			? BCH_COMPRESSION_TYPE_incompressible
 			: op->compression_opt
 			? bch2_bio_compress(c, dst, &dst_len, src, &src_len,
-					    op->compression_opt)
+					    op->compression_opt,
+					    op->pos, !(op->flags & BCH_WRITE_pages_stable))
 			: 0;
 		if (!crc_is_compressed(crc)) {
 			dst_len = min(dst->bi_iter.bi_size, src->bi_iter.bi_size);
@@ -1215,7 +1216,9 @@ static bool bch2_extent_is_writeable(struct bch_write_op *op,
 		if (crc_is_encoded(p.crc) || p.has_ec)
 			return false;
 
-		replicas += bch2_extent_ptr_durability(c, &p);
+		replicas += !p.ptr.cached
+			? bch2_dev_durability(c, p.ptr.dev)
+			: 0;
 	}
 
 	return replicas >= op->opts.data_replicas;
@@ -1281,22 +1284,23 @@ static int bch2_nocow_write_convert_one_unwritten(struct btree_trans *trans,
 static void bch2_nocow_write_convert_unwritten(struct bch_write_op *op)
 {
 	struct bch_fs *c = op->c;
-	struct btree_trans *trans = bch2_trans_get(c);
 	int ret = 0;
 
-	for_each_keylist_key(&op->insert_keys, orig) {
-		ret = for_each_btree_key_max_commit(trans, iter, BTREE_ID_extents,
-				     bkey_start_pos(&orig->k), orig->k.p,
-				     BTREE_ITER_intent, k,
-				     &op->res, NULL,
-				     BCH_TRANS_COMMIT_no_enospc, ({
-			bch2_nocow_write_convert_one_unwritten(trans, &iter, op, orig, k, op->new_i_size);
-		}));
-		if (ret)
-			break;
-	}
+	{
+		CLASS(btree_trans, trans)(c);
 
-	bch2_trans_put(trans);
+		for_each_keylist_key(&op->insert_keys, orig) {
+			ret = for_each_btree_key_max_commit(trans, iter, BTREE_ID_extents,
+					     bkey_start_pos(&orig->k), orig->k.p,
+					     BTREE_ITER_intent, k,
+					     &op->res, NULL,
+					     BCH_TRANS_COMMIT_no_enospc, ({
+				bch2_nocow_write_convert_one_unwritten(trans, &iter, op, orig, k, op->new_i_size);
+			}));
+			if (ret)
+				break;
+		}
+	}
 
 	if (ret && !bch2_err_matches(ret, EROFS)) {
 		struct bkey_i *insert = bch2_keylist_front(&op->insert_keys);
@@ -1344,6 +1348,22 @@ static bool bkey_get_dev_iorefs(struct bch_fs *c, struct bkey_ptrs_c ptrs)
 	return true;
 }
 
+static int bch2_inode_get_i_size(struct btree_trans *trans, struct bpos inode_pos, u64 *i_size)
+{
+	CLASS(btree_iter, iter)(trans, BTREE_ID_inodes, inode_pos, BTREE_ITER_cached);
+	struct bkey_s_c k = bkey_try(bch2_btree_iter_peek_slot(&iter));
+
+	if (likely(k.k->type == KEY_TYPE_inode_v3)) {
+		*i_size = le64_to_cpu(bkey_s_c_to_inode_v3(k).v->bi_size);
+	} else {
+		struct bch_inode_unpacked inode_u;
+		bch2_inode_unpack(k, &inode_u);
+		*i_size = inode_u.bi_size;
+	}
+
+	return 0;
+}
+
 static void bch2_nocow_write(struct bch_write_op *op)
 {
 	struct bch_fs *c = op->c;
@@ -1358,6 +1378,8 @@ static void bch2_nocow_write(struct bch_write_op *op)
 	if (op->flags & BCH_WRITE_move)
 		return;
 
+	op->flags &= ~BCH_WRITE_convert_unwritten;
+
 	trans = bch2_trans_get(c);
 retry:
 	bch2_trans_begin(trans);
@@ -1366,6 +1388,14 @@ retry:
 	if (unlikely(ret))
 		goto err;
 
+	u64 i_size;
+	ret = bch2_inode_get_i_size(trans, SPOS(0, op->pos.inode, snapshot), &i_size);
+	if (unlikely(ret))
+		goto err;
+
+	if (op->new_i_size > i_size)
+		op->flags |= BCH_WRITE_convert_unwritten;
+
 	bch2_trans_iter_init(trans, &iter, BTREE_ID_extents,
 			     SPOS(op->pos.inode, op->pos.offset, snapshot),
 			     BTREE_ITER_slots);
@@ -1426,8 +1456,7 @@ retry:
 		}
 
 		bch2_cut_front(c, op->pos, op->insert_keys.top);
-		if (op->flags & BCH_WRITE_convert_unwritten)
-			bch2_cut_back(POS(op->pos.inode, op->pos.offset + bio_sectors(bio)), op->insert_keys.top);
+		bch2_cut_back(POS(op->pos.inode, op->pos.offset + bio_sectors(bio)), op->insert_keys.top);
 
 		bio = &op->wbio.bio;
 		if (k.k->p.offset < op->pos.offset + bio_sectors(bio)) {
@@ -1450,7 +1479,8 @@ retry:
 		bch2_submit_wbio_replicas(to_wbio(bio), c, BCH_DATA_user,
 					  op->insert_keys.top, true);
 
-		bch2_keylist_push(&op->insert_keys);
+		if (op->flags & BCH_WRITE_convert_unwritten)
+			bch2_keylist_push(&op->insert_keys);
 		if (op->flags & BCH_WRITE_submitted)
 			break;
 		bch2_btree_iter_advance(&iter);
@@ -1555,6 +1585,7 @@ again:
 				&op->devs_have,
 				op->nr_replicas,
 				op->nr_replicas_required,
+				op->opts.data_replicas,
 				op->watermark,
 				op->flags,
 				&op->cl, &wp));
@@ -1576,11 +1607,12 @@ err:
 			op->flags |= BCH_WRITE_submitted;
 
 			if (unlikely(ret < 0)) {
+				op->error = ret;
+
 				/* Extra info on errors from the allocator: */
-				if (!(op->flags & BCH_WRITE_alloc_nowait))
+				if (!(op->flags & BCH_WRITE_move))
 					bch2_write_op_error(op, true, op->pos.offset,
 							    "%s(): %s", __func__, bch2_err_str(ret));
-				op->error = ret;
 				break;
 			}
 		}
@@ -1608,7 +1640,7 @@ err:
 	if ((op->flags & BCH_WRITE_sync) ||
 	    (!(op->flags & BCH_WRITE_submitted) &&
 	     !(op->flags & BCH_WRITE_in_worker))) {
-		bch2_wait_on_allocator(c, &op->cl);
+		bch2_wait_on_allocator(c, op->watermark, &op->cl);
 
 		__bch2_write_index(op);
 
@@ -1777,6 +1809,8 @@ void __bch2_write_op_to_text(struct printbuf *out, struct bch_write_op *op)
 	prt_bitflags(out, bch2_write_flags, op->flags);
 	prt_newline(out);
 
+	prt_printf(out, "watermark:\t%s\n", bch2_watermarks[op->watermark]);
+
 	prt_printf(out, "nr_replicas:\t%u\n", op->nr_replicas);
 	prt_printf(out, "nr_replicas_required:\t%u\n", op->nr_replicas_required);
 	prt_printf(out, "devs_have:\t");
@@ -1804,6 +1838,7 @@ void bch2_write_op_to_text(struct printbuf *out, struct bch_write_op *op)
 
 		prt_str(out, "old key:\t");
 		bch2_bkey_val_to_text(out, u->op.c, bkey_i_to_s_c(u->k.k));
+		prt_newline(out);
 	}
 }
 
