@@ -78,6 +78,7 @@ int bch2_set_version_incompat(struct bch_fs *c, enum bcachefs_metadata_version v
 {
 	if (((c->sb.features & BIT_ULL(BCH_FEATURE_incompat_version_field)) &&
 	     version <= c->sb.version_incompat_allowed)) {
+		guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 		guard(mutex)(&c->sb_lock);
 
 		if (version > c->sb.version_incompat) {
@@ -604,6 +605,9 @@ static void bch2_sb_update(struct bch_fs *c)
 	c->sb.clean		= BCH_SB_CLEAN(src);
 	c->sb.encryption_type	= BCH_SB_ENCRYPTION_TYPE(src);
 
+	c->sb.extent_bp_shift = BCH_SB_EXTENT_BP_SHIFT(c->disk_sb.sb) ?:
+		BCH_SB_EXTENT_BP_SHIFT_DEFAULT;
+
 	c->sb.nsec_per_time_unit = le32_to_cpu(src->time_precision);
 	c->sb.time_units_per_sec = NSEC_PER_SEC / c->sb.nsec_per_time_unit;
 
@@ -1201,7 +1205,7 @@ int bch2_write_super(struct bch_fs *c)
 		scoped_guard(printbuf_indent, &err)
 			bch2_devs_mask_to_text_locked(&err, c, &sb_unwritten);
 
-		if (bch2_fs_emergency_read_only2(c, &err))
+		if (bch2_fs_emergency_read_only(c, &err))
 			bch2_print_str(c, KERN_ERR, err.buf);
 	}
 out:
@@ -1215,6 +1219,7 @@ out:
 
 void __bch2_check_set_feature(struct bch_fs *c, unsigned feat)
 {
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 	if (!(c->sb.features & BIT_ULL(feat))) {
 		c->disk_sb.sb->features[0] |= cpu_to_le64(BIT_ULL(feat));
@@ -1268,6 +1273,7 @@ void bch2_sb_upgrade(struct bch_fs *c, unsigned new_version, bool incompat)
 
 void bch2_sb_upgrade_incompat(struct bch_fs *c)
 {
+	guard(memalloc_flags)(PF_MEMALLOC_NOFS);
 	guard(mutex)(&c->sb_lock);
 
 	if (c->sb.version == c->sb.version_incompat_allowed)
