@@ -811,8 +811,8 @@ int __bch2_unlink(struct inode *vdir, struct dentry *dentry,
 	ret = commit_do(trans, NULL, NULL,
 			BCH_TRANS_COMMIT_no_enospc,
 		bch2_unlink_trans(trans,
-				  inode_inum(dir), &dir_u,
-				  &inode_u, &dentry->d_name,
+				  inode_inum(dir),	&dir_u,
+				  inode_inum(inode),	&inode_u, &dentry->d_name,
 				  deleting_snapshot));
 	if (unlikely(ret))
 		goto err;
@@ -1831,6 +1831,40 @@ static void bch2_evict_inode(struct inode *vinode)
 	if (!delete)
 		bch2_inode_hash_remove(c, inode);
 
+	if (IS_ENABLED(CONFIG_BCACHEFS_DEBUG) && delete)
+		write_inode_now(&inode->v, true);
+
+	if (IS_ENABLED(CONFIG_BCACHEFS_DEBUG) || !delete) {
+		BUG_ON(inode->v.i_state & I_DIRTY);
+
+		struct bch_inode_unpacked inode_u;
+		if (!is_bad_inode(&inode->v) &&
+		    !bch2_journal_error(&c->journal) &&
+		    !bch2_inode_find_by_inum(c, inode_inum(inode), &inode_u)) {
+			if (inode->v.i_size != inode_u.bi_size) {
+				CLASS(bch_log_msg, msg)(c);
+
+				prt_printf(&msg.m, "vfs i_size mismatch at evict: %llu != %llu (inum %llu)\n",
+					   (u64) inode->v.i_size, inode_u.bi_size, inode_u.bi_inum);
+				bch2_trans_do(c, bch2_inum_to_path(trans, inode_inum(inode), &msg.m));
+				prt_newline(&msg.m);
+				bch2_inode_unpacked_to_text(&msg.m, &inode_u);
+				msg.m.suppress = !bch2_count_fsck_err(c, vfs_i_size_bad, &msg.m);
+			}
+
+			if (inode->v.i_blocks != inode_u.bi_sectors) {
+				CLASS(bch_log_msg, msg)(c);
+
+				prt_printf(&msg.m, "vfs i_sectors mismatch at evict: %llu != %llu (inum %llu)\n",
+					   (u64) inode->v.i_blocks, inode_u.bi_sectors, inode_u.bi_inum);
+				bch2_trans_do(c, bch2_inum_to_path(trans, inode_inum(inode), &msg.m));
+				prt_newline(&msg.m);
+				bch2_inode_unpacked_to_text(&msg.m, &inode_u);
+				msg.m.suppress = !bch2_count_fsck_err(c, vfs_i_sectors_bad, &msg.m);
+			}
+		}
+	}
+
 	truncate_inode_pages_final(&inode->v.i_data);
 
 	cancel_delayed_work_sync(&inode->ei_writeback_timer);
@@ -2022,6 +2056,7 @@ static int bch2_show_options(struct seq_file *seq, struct dentry *root)
 	CLASS(printbuf, buf)();
 
 	bch2_opts_to_text(&buf, c->opts, c, c->disk_sb.sb,
+			  &c->mount_opts,
 			  OPT_MOUNT, OPT_HIDDEN, OPT_SHOW_MOUNT_STYLE);
 	printbuf_nul_terminate(&buf);
 	seq_printf(seq, ",%s", buf.buf);
@@ -2105,6 +2140,13 @@ static int bch2_test_super(struct super_block *s, void *data)
 	return true;
 }
 
+static void set_mount_opts(struct bch_fs *c, struct bch_opts *opts)
+{
+	for (enum bch_opt_id id = 0; id < bch2_opts_nr; id++)
+		if (bch2_opt_defined_by_id(opts, id))
+			set_bit(id, c->mount_opts.d);
+}
+
 static int bch2_fs_get_tree(struct fs_context *fc)
 {
 	struct bch_fs *c;
@@ -2141,6 +2183,7 @@ static int bch2_fs_get_tree(struct fs_context *fc)
 
 	if (opt_defined(opts, discard))
 		set_bit(BCH_FS_discard_mount_opt_set, &c->flags);
+	set_mount_opts(c, &opts);
 
 	/* Some options can't be parsed until after the fs is started: */
 	opts = bch2_opts_empty();
@@ -2149,6 +2192,7 @@ static int bch2_fs_get_tree(struct fs_context *fc)
 		goto err_stop_fs;
 
 	bch2_opts_apply(&c->opts, opts);
+	set_mount_opts(c, &opts);
 
 	ret = bch2_fs_start(c);
 	if (ret)
@@ -2333,6 +2377,8 @@ static int bch2_fs_reconfigure(struct fs_context *fc)
 		guard(rwsem_write)(&c->state_lock);
 
 		if (opts->opts.read_only) {
+			try(sync_filesystem(sb));
+
 			bch2_fs_read_only(c);
 
 			sb->s_flags |= SB_RDONLY;
